import { BuilderContext } from '@averjs/builder';
import { ComponentOptions } from 'vue';
import { I18nOptions } from 'vue-i18n';
import { Route } from 'vue-router';
import { RouterOptions } from 'vue-router';
import { Store } from 'vuex';
import { StoreOptions } from 'vuex';
import VueI18n from 'vue-i18n';
import VueRouter from 'vue-router';

export declare type AppEntry<T = any> = (context: AppEntryContext<T>) => UserReturns | void | Promise<void> | Promise<UserReturns>;

export declare interface AppEntryContext<T extends any> extends SSRContext {
    test: string;
    appOptions: {
        [index: string]: any;
        i18n: VueI18n;
        router: VueRouter;
        store: Store<T>;
        ssrContext: SSRContext;
        context: Record<string, unknown>;
        render: ComponentOptions<any, any, any, any>['render'];
    };
}

export declare interface AsyncDataContext<T extends any> {
    app: Vue;
    store: Store<T>;
    route: {
        to: Route;
        from?: Route;
    };
    isServer: boolean;
}

export declare type ClientEntry = (context: ClientEntryContext) => void | Promise<void>;

export declare interface ClientEntryContext {
    userReturns: UserReturns;
}

declare const _default: () => Templates[];
export default _default;

export declare type I18nEntry = (options: I18nOptions) => I18nOptions | Promise<I18nOptions>;

export declare type RouterEntry<T extends any> = (context: RouterEntryContext<T>) => RouterOptions | Promise<RouterOptions>;

export declare interface RouterEntryContext<T extends any> {
    i18n: VueI18n;
    store: Store<T>;
    ssrContext: SSRContext;
    config: RouterOptions;
}

export declare type ServerEntry = (context: ServerEntryContext) => void | Promise<void>;

export declare interface ServerEntryContext extends BuilderContext {
    userReturns: UserReturns;
    contextRendered: (fn: (context: BuilderContext) => void) => void;
}

export declare interface SSRContext {
    isServer: boolean;
    context: BuilderContext;
}

export declare type StoreEntry<T extends any> = (options: StoreOptions<T>) => StoreOptions<T> | Promise<StoreOptions<T>>;

export declare interface Templates {
    src: string;
    dst: string;
    pluginPath?: string;
    dirname?: string;
}

export declare type UserReturns = Record<string, unknown>;

export { }
declare module 'vue/types/vue' {
  interface Vue {
    $locale: {
      change<T>(lang: string | T): void;
      current(): string;
    };
  }
}

declare module 'vue/types/options' {
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  interface ComponentOptions<V extends Vue> {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    asyncData?: <T extends any>(context: AsyncDataContext<T>) => Promise<void>;
  }
}
